/**
 * @file sha1sum.h
 * @brief sha1sum.c Ǵ Լ, ũ, ü  .
 */

#ifndef SHA1SUM_H
#define SHA1SUM_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <errno.h>

#undef BIG_ENDIAN_HOST
typedef unsigned int u32;

/****************
 * Rotate a 32 bit integer by n bytes
 */
#if defined(__GNUC__) && defined(__i386__)
static inline u32 rol( u32 x, int n)
{
	__asm__("roll %%cl,%0"
		:"=r" (x)
		:"0" (x),"c" (n));
	return x;
}
#else
#define rol(x,n) (((x) << (n)) | ((x) >> (32-(n))))
#endif

typedef struct {
	u32  h0,h1,h2,h3,h4;
	u32  nblocks;
	unsigned char buf[64];
	int  count;
} SHA1_CONTEXT;

#define K1	0x5A827999L
#define K2	0x6ED9EBA1L
#define K3	0x8F1BBCDCL
#define K4	0xCA62C1D6L
#define F1(x,y,z)	(z ^ (x & (y ^ z)))
#define F2(x,y,z)	(x ^ y ^ z)
#define F3(x,y,z)	((x & y) | (z & (x | y)))
#define F4(x,y,z)	(x ^ y ^ z)


#define M(i)	(tm = x[i & 0x0f] ^ x[(i - 14) & 0x0f] \
			^ x[(i - 8) & 0x0f] ^ x[( i - 3) & 0x0f] \
			, (x[i & 0x0f] = rol(tm, 1)))

#define R(a, b, c, d, e, f, k, m)  do {e += rol(a, 5)     \
					+ f(b, c, d)  \
					+ k	      \
					+ m;	      \
					b = rol(b, 30);    \
				} while(0)

/****************
 * Transform the message X which consists of 16 32-bit-words
 */
void transform(SHA1_CONTEXT *hd, unsigned char *data);

/* Update the message digest with the contents
 * of INBUF with length INLEN.
 */
void sha1_write(SHA1_CONTEXT *hd, unsigned char *inbuf, size_t inlen);

void sha1_final(SHA1_CONTEXT *hd);

int sha1sum(char *file_name, SHA1_CONTEXT *ctx);

#endif
